unit Regi1632;

{**************************************************************************

  TRegistry1632

  This unit provides access to the registry in Delphi 1.0 AND Delphi 2.0

  Author: Hannes Danzl (e9026733@stud3.tuwien.ac.at)

  FREEWARE

  If you change it, please send me a copy!!

  When running the Application in Windows 3.x the registry-functions do
  nothing!

************************************************************************** }

{$R-}

interface

uses Classes, SysUtils,
     {$ifdef win32}Windows{$else}Wintypes, Winprocs, Reginc{$endif};

type
  ERegistryException = class(Exception);

  TRegistry1632 = class(TObject)
  private
    {$ifdef win32}
    FCurrentKey: HKEY;
    FCurrentUserKey: HKEY;
    {$else}
    FCurrentKey: LongInt;
    FCurrentUserKey: LongInt;
    {$endif}
    FCurrentPath: string;
    FCurrentUserPath: string;
    fRunningWin31:Boolean;
    procedure SetCurrentPath(Value: string);
    procedure SetCurrentUserPath(Value: string);
    function ReadKey(KeyHandle:LongInt;Key, Path:String):string;
    procedure WriteKey(KeyHandle:LongInt;Key, Path, Value:String);
  protected
    property CurrentPath: string read FCurrentPath write SetCurrentPath;
    property CurrentUserPath: string read FCurrentUserPath write SetCurrentUserPath;
  public
    constructor Create;
    destructor Destroy; override;

    { Reads a Value from the registry in HKEY_LOCAL_MACHINE }
    function ReadString(Path, Key:String): string;
    { Reads a Value from the registry in HKEY_CURRENT_USER }
    function ReadUserString(Path, Key:string): string;
    { Writes a Value to the registry in HKEY_LOCAL_MACHINE }
    procedure WriteString(Path, Key, Value:String);
    { Writes a Value from the registry in HKEY_CURRENT_USER }
    procedure WriteUserString(Path, Key, Value:String);

    property RunningWin31:Boolean read FRunningWin31;
  end;

implementation

uses dialogs;

constructor TRegistry1632.Create;
var achar:array[0..256] of char;
    along1,along2,along:longint;
    mainversion, subversion:longint;
begin
  inherited Create;

  frunningwin31:=false;
  {$ifndef win32}
  { check for windowsnt or wind95 }
  if (GetWinFlags and $4000 {WF_WINNT} ) = 0 then
    if lobyte(loword(getversion))<4 then
      frunningwin31:=true;
  {$endif}    

  FCurrentUserKey:=0;
  FCurrentKey:=0;
  strpcopy(achar,'');

  if runningwin31 then
    exit;

  if RegOpenKeyEx(HKEY_CURRENT_USER,nil,0,
    KEY_ALL_ACCESS, fCurrentUserKey)<>error_success then
    raise ERegistryException.Create('RegOpenKey failed!');
  if RegOpenKeyEx(HKEY_LOCAL_MACHINE,nil,0, KEY_ALL_ACCESS, fCurrentKey)<>error_success then
    raise ERegistryException.Create('RegOpenKey failed!');
end;

destructor TRegistry1632.Destroy;
begin
  if FCurrentUserKey<>0 then
    regCloseKey(FCurrentUserKey);
  if FCurrentKey<>0 then
    regCloseKey(FCurrentKey);
  inherited Destroy;
end;

procedure TRegistry1632.WriteString(Path, Key, Value:String);
begin
  CurrentPath:=Path;
  WriteKey(fCurrentKey,Key,path, Value);
end;

procedure TRegistry1632.WriteUserString(Path, Key, Value:String);
begin
  CurrentUserPath:=Path;
  WriteKey(fCurrentUserKey,Key,path, Value);
end;

procedure TRegistry1632.WriteKey(KeyHandle:LongInt;Key, Path, Value:String);
var
  aValue:array[0..256] of char;
  aname:array[0..256] of char;
  DataType:Longint;
  DataLen:longint;
  QueryResult:LongInt;
begin
  if runningwin31 then
    exit;
  strpcopy(avalue,value);
  DataType := REG_SZ;
  DataLen:=strlen(avalue);

  strpcopy(aname,key);
  QueryResult:=RegSetValueEx(KeyHandle, @aName, 0, DataType, PByte(@aValue),
    DataLen);
  if QueryResult <> ERROR_SUCCESS then
      raise ERegistryException.CreateFmt('RegSetValue failed: %s set to %s', [Path+Key, Value]);
end;

function TRegistry1632.ReadString(Path: string; Key:String): string;
begin
  CurrentUserPath:=Path;
  Result:=ReadKey(fCurrentUserKey,Key, path);
end;

function TRegistry1632.ReadUserString(Path: string; Key:string): string;
begin
  CurrentUserPath:=Path;
  Result:=ReadKey(fCurrentUserKey,Key, path);
end;

function TRegistry1632.ReadKey(KeyHandle:LongInt;Key, Path:String):string;
var
  aresult:array[0..256] of char;
  aname:array[0..256] of char;
{$ifdef win32}
  bufsize:Integer;
  DataType:Integer;
{$else}
  bufsize:Longint;
  DataType:Longint;
{$endif}
  QueryResult:longint;
begin
  if runningwin31 then
  begin
    result:='Registry not supported when running Windows 3.x';
    exit;
  end;

  DataType := REG_SZ;
  BufSize:=255;
  strpcopy(aname,key);
{$ifndef win32}  
  QueryResult:=RegQueryValueEx(KeyHandle, @aName, nil, @DataType, PByte(@aresult),
    BufSize);
{$else}
  QueryResult:=RegQueryValueEx(KeyHandle, @aName, nil, pDWord(@DataType), PByte(@aresult),
    pDWord(@BufSize));
{$endif}
  if QueryResult <> ERROR_SUCCESS then
      raise ERegistryException.CreateFmt('RegQueryValue failed: %s', [Path+Key]);
  result:=strpas(aresult);
end;

procedure TRegistry1632.SetCurrentPath(Value: string);
var aname:array[0..256] of char;
begin
  fCurrentPath:=value;
  if runningwin31 then
    exit;

  if FcurrentKey<>0 then
    RegCloseKey(FCurrentKey);
  strpcopy(aname,Value);
  if RegOpenKeyEx(HKEY_LOCAL_MACHINE,aname,0,
    KEY_ALL_ACCESS, fCurrentKey)<>error_success then
    raise ERegistryException.Create('RegOpenKey failed!');
end;

procedure TRegistry1632.SetCurrentUserPath(Value: string);
var aname:array[0..256] of char;
begin
  fCurrentPath:=value;
  if runningwin31 then
    exit;

  if FcurrentUserKey<>0 then
    RegCloseKey(FCurrentUserKey);
  strpcopy(aname,Value);
  if RegOpenKeyEx(HKEY_CURRENT_USER,aname,0,
    KEY_ALL_ACCESS, fCurrentUserKey)<>error_success then
    raise ERegistryException.Create('RegOpenKey failed!');
end;

end.


